@@ -1,12 +1,12 @@
 # Purpose        : Fit a 2D or 3D regression model;
 # Maintainer     : Tomislav Hengl (tom.hengl@wur.nl)
-# Contributions  : Bas Kempen (bas.kempen@wur.nl) and Gerard B.M. Heuvelink (gerard.heuvelink@wur.nl); 
+# Contributions  : Bas Kempen (bas.kempen@wur.nl) and Gerard B.M. Heuvelink (gerard.heuvelink@wur.nl) and Mario Antonio Guevara (mguevara@udel.edu); 
 # Dev Status     : Alpha
 # Note           : Regression families considered spatial GLMs, CART, random forest, linear mixed-effect models ...;
 
 
 ## Fit a GLM to spatial data:
-setMethod("fit.regModel", signature(formulaString = "formula", rmatrix = "data.frame", predictionDomain = "SpatialPixelsDataFrame", method = "character"), function(formulaString, rmatrix, predictionDomain, method = list("GLM", "rpart", "randomForest", "quantregForest", "lme")[[1]], dimensions = NULL, fit.family = gaussian(), stepwise = TRUE, rvgm, GLS = FALSE, random, steps = 100, ...){
+setMethod("fit.regModel", signature(formulaString = "formula", rmatrix = "data.frame", predictionDomain = "SpatialPixelsDataFrame", method = "character"), function(formulaString, rmatrix, predictionDomain, method = list("GLM", "rpart", "randomForest", "quantregForest", "lme")[[1]], dimensions = NULL, fit.family = gaussian(), stepwise = TRUE, rvgm, GLS = FALSE, random, steps = 100, subsample, subsample.reg, ...){                         
 
   ## parent call:
   parent_call <- as.list(substitute(list(...)))[-1]
@@ -16,6 +16,10 @@ setMethod("fit.regModel", signature(formulaString = "formula", rmatrix = "data.f
   if(!any(names(rmatrix) %in% tv)){
     stop("Target variable not found in the 'rmatrix' object.")
   }
+  ## reserved names:
+  if(any(names(rmatrix) %in% paste(tv, "residual", sep="."))){
+    warning(paste0("Name mismatch in the rmatrix: '", tv, ".residual' already in use"))
+  }
   
   ## spatial coordinates (column names):
   xyn <- attr(predictionDomain@bbox, "dimnames")[[1]]
@@ -31,10 +35,22 @@ setMethod("fit.regModel", signature(formulaString = "formula", rmatrix = "data.f
   ## check if the method exists:
   if(length(method)>1){ stop("'method' argument contains multiple options") }
   if(!any(method %in% list("GLM", "rpart", "randomForest", "quantregForest", "lme"))){ stop(paste(method, "method not available.")) }
-    
+  
+  ## subsample regression if necessary:
+  s <- 1:nrow(rmatrix)
+  if(!missing(subsample.reg)){
+     if(nrow(rmatrix) > subsample.reg){
+       message(paste0("Subsetting regression matrix to ", subsample.reg, " observations..."))
+       s <- sample(1:nrow(rmatrix), subsample.reg)
+     }
+  }
+  rmatrix.s <- rmatrix[s,]
+   
+  ## 1. REGRESSION MODEL FITTING
   if(method == "lme" | !missing(random)){
     message("Fitting a Mixel-effect linear model...")
     if(requireNamespace("nlme", quietly = TRUE)){
+      if(!missing(subsample.reg)){ message("Ignoring 'subsample.reg' argument...") }
       ## check if the random component is defined:
       if(!missing(random)){
         rgm <- nlme::lme(formulaString, random=random, data=rmatrix, na.action=na.omit)
@@ -43,7 +59,9 @@ setMethod("fit.regModel", signature(formulaString = "formula", rmatrix = "data.f
       }
       ## extract the residuals:
       if(any(names(rgm) == "na.action")){  rmatrix <- rmatrix[-rgm$na.action,] }
-      rmatrix$residual <- resid(rgm)
+      rmatrix[,paste(tv, "residual", sep=".")] <- resid(rgm)
+    } else {
+      stop("Package 'nlme' not available")
     }
   }
   
@@ -53,18 +71,19 @@ setMethod("fit.regModel", signature(formulaString = "formula", rmatrix = "data.f
       if(!dimensions == "2D"){ stop("Fitting of the models using the GLS option possible with '2D' data only") }
       message("Fitting a LM using Generalized Least Squares...")
       if(requireNamespace("nlme", quietly = TRUE)){
-        rgm <- nlme::gls(formulaString, rmatrix, correlation=corExp(nugget=TRUE), na.action=na.omit)
+        rgm <- nlme::gls(formulaString, rmatrix.s, correlation=corExp(nugget=TRUE), na.action=na.omit)
         ## extract the residuals:
-        if(any(names(rgm) == "na.action")){  rmatrix <- rmatrix[-rgm$na.action,] }
-        rmatrix$residual <- resid(rgm)
+        rmatrix[,paste(tv, "residual", sep=".")] <- rmatrix[,tv] - predict(rgm, rmatrix)
+      } else {
+      stop("Package 'nlme' not available")
       }
     } else {
+      if(!missing(subsample.reg)){ message("Ignoring 'subsample.reg' argument...") }
       if(all(c("family","link") %in% names(fit.family))){
         if(fit.family$family=="gaussian"&fit.family$link=="identity"){
           message("Fitting a linear model...")
         } 
-      }
-      else {  
+      } else {  
         message("Fitting a GLM...")
       }
       if(any(names(parent_call) %in% "weights")){ 
@@ -75,12 +94,10 @@ setMethod("fit.regModel", signature(formulaString = "formula", rmatrix = "data.f
       }
       if(stepwise == TRUE){
         rgm <- step(rgm, trace=0, steps=steps)
-      }
-   
-      ## mask out the missing values:
+      }      
+      ## extract the response residuals: [http://stackoverflow.com/questions/2531489/understanding-glmresiduals-and-residglm]
       if(any(names(rgm) == "na.action")){  rmatrix <- rmatrix[-rgm$na.action,] }
-      ## extract the response residuals: [http://stackoverflow.com/questions/2531489/understanding-glmresiduals-and-residglm] 
-        rmatrix$residual <- resid(rgm, type="response")
+      rmatrix[,paste(tv, "residual", sep=".")] <- resid(rgm, type="response") 
     }
   }
   
@@ -88,7 +105,7 @@ setMethod("fit.regModel", signature(formulaString = "formula", rmatrix = "data.f
     ## fit/filter the regression model:
     message("Fitting a regression tree model...")
     if(requireNamespace("rpart", quietly = TRUE)){
-      rgm <- rpart::rpart(formulaString, data=rmatrix)
+      rgm <- rpart::rpart(formulaString, data=rmatrix.s)
       if(stepwise == TRUE){
         ## TH: "A good choice of cp for pruning is often the leftmost value for which the mean lies below the horizontal line"
         ## BK: determine row in complexity table with smallest xerror:
@@ -109,9 +126,8 @@ setMethod("fit.regModel", signature(formulaString = "formula", rmatrix = "data.f
         message(paste("Estimated Complexity Parameter (for prunning):", signif(cpar, 4)))
         rgm <- rpart::prune(rgm, cp=cpar)
       }  
-      ## extract the residuals:
-      if(any(names(rgm) == "na.action")){  rmatrix <- rmatrix[-rgm$na.action,] } 
-      rmatrix$residual <- resid(rgm)  
+    } else {
+      stop("Package 'rpart' not available")
     }
   }
   
@@ -120,32 +136,47 @@ setMethod("fit.regModel", signature(formulaString = "formula", rmatrix = "data.f
       ## fit/filter the regression model:
       message("Fitting a randomForest model...")
       ## NA's not permitted and need to be filtered out:
-      f <- stats::complete.cases(rmatrix[,all.vars(formulaString)])
-      rmatrix <- rmatrix[f,]    
+      f <- stats::complete.cases(rmatrix.s[,all.vars(formulaString)])
+      rmatrix.s <- rmatrix.s[f,]    
       if(method == "randomForest"){
-        rgm <- randomForest::randomForest(formulaString, data=rmatrix, importance=TRUE, na.action=na.omit)
+        if(any(names(parent_call) %in% "mtry")){
+          mtry <- eval(parent_call[["mtry"]])
+          rgm <- randomForest::randomForest(formulaString, data=rmatrix.s, importance=TRUE, na.action=na.omit, mtry=mtry)
+        } else {
+          rgm <- randomForest::randomForest(formulaString, data=rmatrix.s, importance=TRUE, na.action=na.omit)
+        }
       } else {
         ## TH: the quantreg package developed by Nicolai Meinshausen <meinshausen@stats.ox.ac.uk> is slower but more flexible      
         if(requireNamespace("quantregForest", quietly = TRUE)){
-          rgm <- quantregForest::quantregForest(y=eval(formulaString[[2]], rmatrix), x=rmatrix[,all.vars(formulaString)[-1]])
+          rgm <- quantregForest::quantregForest(y=eval(formulaString[[2]], rmatrix.s), x=rmatrix.s[,all.vars(formulaString)[-1]])
           attr(rgm$y, "name") <- tv
         }  
       }
-      ## extract the residuals:
-      rmatrix$residual <- rgm$predicted - rgm$y
+    } else {
+      stop("Package 'randomForest' not available")
     }
   }
   
+  ## Extract residuals:
+  if(method == "quantregForest"){
+    ## breaks if there are incomplete obs:
+    f0 <- which(stats::complete.cases(rmatrix[,all.vars(formulaString)]))
+    rmatrix[f0,paste(tv, "residual", sep=".")] <- rmatrix[f0,tv] - predict(rgm, newdata=rmatrix[f0,attr(rgm$forest$ncat, "names")], quantiles=.5)
+  } 
+  if(method == "randomForest"|method == "rpart"){
+    rmatrix[,paste(tv, "residual", sep=".")] <- rmatrix[,tv] - predict(rgm, newdata=rmatrix, na.action = na.pass)
+  }
+  
   ## TH: here we will add more regression models...
   
   ## test the normality of residuals:
-  if(length(rmatrix$residual)>4999){
+  if(length(rmatrix[,paste(tv, "residual", sep=".")])>4999){
     # subset residuals if necessary...
-    x = rmatrix$residual[runif(length(rmatrix$residual))<4000/length(rmatrix$residual)]
+    x = rmatrix[sample(1:nrow(rmatrix), 4999), paste(tv, "residual", sep=".")]
   } else {
-    x = rmatrix$residual
+    x = rmatrix[,paste(tv, "residual", sep=".")]
   }
-  st = shapiro.test(x)
+  st <- shapiro.test(x)
   if(st$p.value < 0.05|is.na(st$p.value)){
     ## try second test:
     if(requireNamespace("nortest", quietly = TRUE)){
@@ -156,24 +187,33 @@ setMethod("fit.regModel", signature(formulaString = "formula", rmatrix = "data.f
     }
   }
 
+  ## 2. VARIOGRAM FITTING
+  if(any(names(parent_call) %in% "cutoff")){
+     cutoff <- eval(parent_call[["cutoff"]])
+  } else {
+     cutoff <- NULL
+  }
+  if(missing(subsample)){
+    subsample <- nrow(rmatrix)
+  }
   if(missing(rvgm)&GLS==FALSE){
   ## If variogram is not defined, try to fit variogram 2D or 3D data:
     if(dimensions == "2D"){ 
       message("Fitting a 2D variogram...")
-      rvgm <- fit.vgmModel(residual ~ 1, rmatrix = rmatrix, predictionDomain = predictionDomain, dimensions = "2D") 
+      rvgm <- fit.vgmModel(as.formula(paste0(tv, ".residual ~ 1")), rmatrix = rmatrix, predictionDomain = predictionDomain, dimensions = "2D", subsample=subsample, cutoff=cutoff) 
     }
     if(dimensions == "3D"){ 
       message("Fitting a 3D variogram...")
-      rvgm <- fit.vgmModel(residual ~ 1, rmatrix = rmatrix, predictionDomain = predictionDomain, dimensions = "3D") 
+      rvgm <- fit.vgmModel(as.formula(paste0(tv, ".residual ~ 1")), rmatrix = rmatrix, predictionDomain = predictionDomain, dimensions = "3D", subsample=subsample, cutoff=cutoff) 
     }
     } else {
       ## TH: The nlme package fits a variogram, but this is difficult to translate to gstat format:
       if(missing(rvgm)&any(class(rgm)=="gls")){
-           rvgm <- fit.vgmModel(residual ~ 1, rmatrix = rmatrix, predictionDomain = predictionDomain, dimensions = "2D")
+           rvgm <- fit.vgmModel(as.formula(paste0(tv, ".residual ~ 1")), rmatrix = rmatrix, predictionDomain = predictionDomain, dimensions = "2D", subsample=subsample, cutoff=cutoff)
       } else { 
         ## Use a pure nugget effect if variogram is set to NULL
         if(is.null(rvgm)){
-          rvgm <- fit.vgmModel(residual ~ 1, rmatrix = rmatrix, predictionDomain = predictionDomain, dimensions = dimensions, vgmFun = "Nug")
+          rvgm <- fit.vgmModel(as.formula(paste0(tv, ".residual ~ 1")), rmatrix = rmatrix, predictionDomain = predictionDomain, dimensions = dimensions, vgmFun = "Nug", subsample=subsample)
         } else {
           xyn = attr(predictionDomain@bbox, "dimnames")[[1]]
           ## create spatial points:
@@ -181,7 +221,8 @@ setMethod("fit.regModel", signature(formulaString = "formula", rmatrix = "data.f
           proj4string(rmatrix) = predictionDomain@proj4string
           observations = as(rmatrix, "SpatialPoints")
           ## othewise copy the variogram submitted by the user:
-          rvgm <- list(vgm=rvgm, observations=observations, svgm=NA)
+          svgm <- gstat::variogram(formulaString, rmatrix)
+          rvgm <- list(vgm=rvgm, observations=observations, svgm=svgm)
           }
        }
   }
